"""Unit tests for validation utilities."""

import os
import sys
import unittest
from unittest.mock import patch, MagicMock, mock_open
from pathlib import Path
import json

# Add the project root to the Python path
sys.path.insert(0, str(Path(__file__).parent.parent))

from dolze_templates.exceptions import ValidationError, ResourceError
from dolze_templates.utils.validation import (
    validate_type,
    validate_range,
    validate_choice,
    validate_color,
    validate_position,
    validate_size,
    validate_font_path,
    validate_url,
    validate_component,
    validate_template_config,
    validate_text_component,
    validate_image_component,
    validate_shape_component,
    MIN_FONT_SIZE,
    MAX_FONT_SIZE,
    VALID_FONT_WEIGHTS,
    VALID_TEXT_ALIGNS,
    VALID_BORDER_STYLES,
    VALID_EFFECTS,
    MIN_IMAGE_DIMENSION,
    MAX_IMAGE_DIMENSION,
    MIN_BLUR_RADIUS,
    MAX_BLUR_RADIUS,
)


class TestValidationUtils(unittest.TestCase):
    """Test validation utility functions."""

    def test_validate_type_valid(self):
        """Test validate_type with valid input."""
        self.assertEqual(validate_type(42, int, "test"), 42)
        self.assertEqual(validate_type("test", str, "test"), "test")
        self.assertEqual(validate_type(3.14, (int, float), "test"), 3.14)
        self.assertEqual(validate_type(True, bool, "test"), True)
        self.assertEqual(validate_type(None, type(None), "test"), None)

    def test_validate_type_invalid(self):
        """Test validate_type with invalid input."""
        with self.assertRaises(ValidationError) as cm:
            validate_type("not an int", int, "test")
        self.assertIn("must be of type int", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_type(42, str, "test")
        self.assertIn("must be of type str", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_type(None, int, "test", allow_none=False)
        self.assertIn("must be of type int", str(cm.exception))

    def test_validate_range_valid(self):
        """Test validate_range with valid input."""
        self.assertEqual(validate_range(5, 0, 10, "test"), 5)
        self.assertEqual(validate_range(0, 0, 10, "test"), 0)
        self.assertEqual(validate_range(10, 0, 10, "test"), 10)
        self.assertEqual(validate_range(5.5, 0.0, 10.0, "test"), 5.5)
        self.assertEqual(validate_range(-5, -10, 10, "test"), -5)

    def test_validate_range_invalid(self):
        """Test validate_range with invalid input."""
        with self.assertRaises(ValidationError) as cm:
            validate_range(15, 0, 10, "test")
        self.assertIn("must be between 0 and 10", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_range(-1, 0, 10, "test")
        self.assertIn("must be between 0 and 10", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_range("not a number", 0, 10, "test")
        self.assertIn("must be a number", str(cm.exception))

    def test_validate_choice_valid(self):
        """Test validate_choice with valid input."""
        choices = ["a", "b", "c"]
        self.assertEqual(validate_choice("a", choices, "test"), "a")
        self.assertEqual(validate_choice("b", choices, "test"), "b")
        self.assertEqual(validate_choice("c", choices, "test"), "c")
        self.assertEqual(validate_choice(None, choices, "test", allow_none=True), None)

    def test_validate_choice_invalid(self):
        """Test validate_choice with invalid input."""
        choices = ["a", "b", "c"]
        with self.assertRaises(ValidationError) as cm:
            validate_choice("d", choices, "test")
        self.assertIn("must be one of", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_choice(123, choices, "test")
        self.assertIn("must be one of", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_choice(None, choices, "test", allow_none=False)
        self.assertIn("must be one of", str(cm.exception))

    def test_validate_color_valid(self):
        """Test validate_color with valid input."""
        # Test hex colors
        self.assertEqual(validate_color("#ff0000", "test"), (255, 0, 0, 255))
        self.assertEqual(validate_color("#00ff00", "test"), (0, 255, 0, 255))
        self.assertEqual(validate_color("#0000ff", "test"), (0, 0, 255, 255))
        self.assertEqual(
            validate_color("#ff000080", "test"), (255, 0, 0, 128)
        )  # With alpha
        self.assertEqual(validate_color("#f00", "test"), (255, 0, 0, 255))  # Short hex
        self.assertEqual(
            validate_color("#f008", "test"), (255, 0, 0, 136)
        )  # Short hex with alpha

        # Test RGB tuples
        self.assertEqual(validate_color((255, 0, 0), "test"), (255, 0, 0, 255))
        self.assertEqual(validate_color([0, 255, 0], "test"), (0, 255, 0, 255))

        # Test RGBA tuples
        self.assertEqual(validate_color((255, 0, 0, 128), "test"), (255, 0, 0, 128))
        self.assertEqual(validate_color([0, 255, 0, 64], "test"), (0, 255, 0, 64))

        # Test None
        self.assertEqual(validate_color(None, "test"), (0, 0, 0, 0))  # Transparent

        # Test with alpha=False
        self.assertEqual(
            validate_color((255, 0, 0), "test", alpha=False), (255, 0, 0, 255)
        )

    def test_validate_color_invalid(self):
        """Test validate_color with invalid input."""
        # Invalid hex
        with self.assertRaises(ValidationError) as cm:
            validate_color("not a color", "test")
        self.assertIn("Invalid color format", str(cm.exception))

        # Invalid hex format
        with self.assertRaises(ValidationError) as cm:
            validate_color("#zzz", "test")
        self.assertIn("Invalid color format", str(cm.exception))

        # Invalid RGB values
        with self.assertRaises(ValidationError) as cm:
            validate_color((300, 0, 0), "test")
        self.assertIn("Color values must be between 0 and 255", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_color((0, -1, 0), "test")
        self.assertIn("Color values must be between 0 and 255", str(cm.exception))

        # Invalid alpha
        with self.assertRaises(ValidationError) as cm:
            validate_color((0, 0, 255, 300), "test")
        self.assertIn("Alpha value must be between 0 and 255", str(cm.exception))

        # Invalid type
        with self.assertRaises(ValidationError) as cm:
            validate_color(12345, "test")
        self.assertIn(
            "Color must be a hex string or RGBA tuple/list", str(cm.exception)
        )

        # Invalid sequence length
        with self.assertRaises(ValidationError) as cm:
            validate_color((1, 2), "test")
        self.assertIn("Color must have 3 or 4 values", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_color((1, 2, 3, 4, 5), "test")
        self.assertIn("Color must have 3 or 4 values", str(cm.exception))

    def test_validate_position_valid(self):
        """Test validate_position with valid input."""
        # Test with dict
        position = {"x": 10, "y": 20}
        self.assertEqual(validate_position(position, "test"), (10, 20))

        # Test with dict with string numbers
        position_str = {"x": "10", "y": "20"}
        self.assertEqual(validate_position(position_str, "test"), (10, 20))

        # Test with tuple
        self.assertEqual(validate_position((30, 40), "test"), (30, 40))

        # Test with list
        self.assertEqual(validate_position([50, 60], "test"), (50, 60))

        # Test with float values (should be converted to int)
        self.assertEqual(validate_position((10.5, 20.9), "test"), (10, 20))

        # Test with string values
        self.assertEqual(validate_position(("30", "40"), "test"), (30, 40))

    def test_validate_position_invalid(self):
        """Test validate_position with invalid input."""
        # Missing keys
        with self.assertRaises(ValidationError) as cm:
            validate_position({"x": 10}, "test")
        self.assertIn("must contain x and y coordinates", str(cm.exception))

        # Invalid type
        with self.assertRaises(ValidationError) as cm:
            validate_position("not a position", "test")
        self.assertIn(
            "must be a dict with x,y or a sequence of 2 numbers", str(cm.exception)
        )

        # Invalid sequence length
        with self.assertRaises(ValidationError) as cm:
            validate_position([10], "test")
        self.assertIn("must be a sequence of 2 numbers", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_position([10, 20, 30], "test")
        self.assertIn("must be a sequence of 2 numbers", str(cm.exception))

        # Non-numeric values
        with self.assertRaises(ValidationError) as cm:
            validate_position(["not", "numbers"], "test")
        self.assertIn("must contain numbers", str(cm.exception))

    def test_validate_size_valid(self):
        """Test validate_size with valid input."""
        # Test with dict
        size = {"width": 100, "height": 200}
        self.assertEqual(validate_size(size, "test"), (100, 200))

        # Test with dict with string numbers
        size_str = {"width": "100", "height": "200"}
        self.assertEqual(validate_size(size_str, "test"), (100, 200))

        # Test with tuple
        self.assertEqual(validate_size((300, 400), "test"), (300, 400))

        # Test with list
        self.assertEqual(validate_size([500, 600], "test"), (500, 600))

        # Test with float values (should be converted to int)
        self.assertEqual(validate_size((100.5, 200.9), "test"), (100, 200))

        # Test with string values
        self.assertEqual(validate_size(("300", "400"), "test"), (300, 400))

        # Test with min/max constraints
        self.assertEqual(
            validate_size((MIN_IMAGE_DIMENSION, MAX_IMAGE_DIMENSION), "test"),
            (MIN_IMAGE_DIMENSION, MAX_IMAGE_DIMENSION),
        )

    def test_validate_size_invalid(self):
        """Test validate_size with invalid input."""
        # Missing keys
        with self.assertRaises(ValidationError) as cm:
            validate_size({"width": 100}, "test")
        self.assertIn("must contain width and height", str(cm.exception))

        # Invalid type
        with self.assertRaises(ValidationError) as cm:
            validate_size("not a size", "test")
        self.assertIn(
            "must be a dict with width,height or a sequence of 2 numbers",
            str(cm.exception),
        )

        # Invalid sequence length
        with self.assertRaises(ValidationError) as cm:
            validate_size([100], "test")
        self.assertIn("must be a sequence of 2 numbers", str(cm.exception))

        with self.assertRaises(ValidationError) as cm:
            validate_size([100, 200, 300], "test")
        self.assertIn("must be a sequence of 2 numbers", str(cm.exception))

        # Non-numeric values
        with self.assertRaises(ValidationError) as cm:
            validate_size(["not", "numbers"], "test")
        self.assertIn("must contain numbers", str(cm.exception))

        # Invalid dimensions
        with self.assertRaises(ValidationError) as cm:
            validate_size([0, 100], "test")
        self.assertIn(
            f"width must be between {MIN_IMAGE_DIMENSION} and {MAX_IMAGE_DIMENSION}",
            str(cm.exception),
        )

        with self.assertRaises(ValidationError) as cm:
            validate_size([100, 0], "test")
        self.assertIn(
            f"height must be between {MIN_IMAGE_DIMENSION} and {MAX_IMAGE_DIMENSION}",
            str(cm.exception),
        )

        with self.assertRaises(ValidationError) as cm:
            validate_size([MAX_IMAGE_DIMENSION + 1, 100], "test")
        self.assertIn(
            f"width must be between {MIN_IMAGE_DIMENSION} and {MAX_IMAGE_DIMENSION}",
            str(cm.exception),
        )

        # Test with percentage
        size_dict_pct = {"width": "50%", "height": "100%"}
        self.assertEqual(
            validate_size(size_dict_pct, "test"),
            {"width": (50, "%"), "height": (100, "%")},
        )

    @patch("os.path.isfile")
    @patch("PIL.ImageFont.truetype")
    def test_validate_font_path_valid(self, mock_truetype, mock_isfile):
        """Test validate_font_path with valid font path."""
        mock_isfile.return_value = True
        mock_truetype.return_value = MagicMock()

        # Test with existing font file
        result = validate_font_path("/path/to/font.ttf")
        self.assertEqual(result, "/path/to/font.ttf")

        # Test with font name
        result = validate_font_path("Arial")
        self.assertEqual(result, "Arial")

    def test_validate_url_valid(self):
        """Test validate_url with valid URLs."""
        urls = [
            "https://example.com",
            "http://example.com/path?query=string",
            "https://sub.example.com:8080/path#fragment",
        ]
        for url in urls:
            self.assertEqual(validate_url(url, "test"), url)

    def test_validate_component_text(self):
        """Test validate_component with a text component."""
        component = {
            "type": "text",
            "text": "Hello, World!",
            "position": {"x": 10, "y": 20},
            "font_size": 24,
            "color": "#ff0000",
            "font_weight": "bold",
            "align": "center",
        }

        result = validate_component(component)
        self.assertEqual(result["type"], "text")
        self.assertEqual(result["text"], "Hello, World!")
        self.assertEqual(result["font_size"], 24)
        self.assertEqual(result["color"], (255, 0, 0, 255))
        self.assertEqual(result["font_weight"], "bold")
        self.assertEqual(result["align"], "center")

    def test_validate_component_image(self):
        """Test validate_component with an image component."""
        component = {
            "type": "image",
            "image_url": "https://example.com/image.jpg",
            "position": {"x": 0, "y": 0},
            "size": {"width": 100, "height": 100},
        }

        with patch(
            "dolze_templates.utils.validation.validate_url"
        ) as mock_validate_url:
            mock_validate_url.return_value = "https://example.com/image.jpg"
            result = validate_component(component)

        self.assertEqual(result["type"], "image")
        self.assertEqual(result["image_url"], "https://example.com/image.jpg")
        self.assertEqual(result["size"], {"width": 100, "height": 100})

    def test_validate_template_config_minimal(self):
        """Test validate_template_config with minimal valid config."""
        config = {
            "name": "Test Template",
            "components": [
                {"type": "text", "text": "Hello, World!", "position": {"x": 0, "y": 0}}
            ],
        }

        with patch(
            "dolze_templates.utils.validation.validate_component"
        ) as mock_validate_component:
            mock_validate_component.return_value = config["components"][0].copy()
            result = validate_template_config(config)

        self.assertEqual(result["name"], "Test Template")
        self.assertEqual(len(result["components"]), 1)
        self.assertEqual(result["components"][0]["type"], "text")

    def test_validate_template_config_full(self):
        """Test validate_template_config with all possible fields."""
        config = {
            "name": "Full Template",
            "size": {"width": 800, "height": 600},
            "background_color": "#ffffff",
            "use_base_image": False,
            "effects": {"blur": {"radius": 5}},
            "components": [
                {
                    "type": "text",
                    "text": "Title",
                    "position": {"x": 10, "y": 20},
                    "font_size": 32,
                    "color": "#000000",
                },
                {
                    "type": "image",
                    "image_url": "https://example.com/logo.png",
                    "position": {"x": 100, "y": 100},
                    "size": {"width": 200, "height": 100},
                },
            ],
        }

        with patch(
            "dolze_templates.utils.validation.validate_component"
        ) as mock_validate_component, patch(
            "dolze_templates.utils.validation.validate_url"
        ) as mock_validate_url:

            # Mock validate_component to return the component as-is
            def mock_validate(component, index):
                return component

            mock_validate_component.side_effect = mock_validate
            mock_validate_url.return_value = "https://example.com/logo.png"

            result = validate_template_config(config)

        self.assertEqual(result["name"], "Full Template")
        self.assertEqual(result["size"], {"width": 800, "height": 600})
        self.assertEqual(result["background_color"], (255, 255, 255, 255))
        self.assertFalse(result["use_base_image"])
        self.assertEqual(len(result["components"]), 2)
        self.assertEqual(result["components"][0]["type"], "text")
        self.assertEqual(result["components"][1]["type"], "image")


if __name__ == "__main__":
    unittest.main()



import os
import json
import unittest
import tempfile
from pathlib import Path
from unittest.mock import MagicMock, patch, mock_open

# Add parent directory to path so we can import the package
import sys

sys.path.append(str(Path(__file__).parent.parent))

from dolze_templates.core import (
    Template,
    TemplateEngine,
    TemplateRegistry,
    get_template_registry,
    FontManager,
    get_font_manager,
)
from dolze_templates.components import TextComponent, ImageComponent
from dolze_templates.utils.validation import validate_template_config


class TestTemplate(unittest.TestCase):
    "Test case for the Template class."

    def setUp(self):
        "Set up test fixtures."
        self.components = [
            TextComponent(
                "Test Text", position=(10, 10), font_size=24, color=(0, 0, 0, 255)
            ),
            ImageComponent("test.jpg", position=(100, 100), size=(200, 200)),
        ]

        self.template = Template(
            name="test_template",
            size=(800, 600),
            background_color=(255, 255, 255, 255),
            components=self.components,
            use_base_image=False,
        )

    def test_template_initialization(self):
        "Test that a template can be initialized."
        self.assertEqual(self.template.name, "test_template")
        self.assertEqual(self.template.size, (800, 600))
        self.assertEqual(self.template.background_color, (255, 255, 255, 255))
        self.assertEqual(len(self.template.components), 2)
        self.assertFalse(self.template.use_base_image)

    def test_add_component(self):
        "Test adding a component to the template."
        new_component = TextComponent("New Text", position=(50, 50))
        self.template.add_component(new_component)

        self.assertEqual(len(self.template.components), 3)
        self.assertEqual(self.template.components[-1], new_component)

    def test_remove_component(self):
        "Test removing a component from the template."
        component_to_remove = self.components[0]
        self.template.remove_component(component_to_remove)

        self.assertEqual(len(self.template.components), 1)
        self.assertNotIn(component_to_remove, self.template.components)

    def test_to_dict(self):
        "Test converting a template to a dictionary."
        template_dict = self.template.to_dict()

        self.assertEqual(template_dict["name"], "test_template")
        self.assertEqual(template_dict["size"], {"width": 800, "height": 600})
        self.assertEqual(template_dict["background_color"], [255, 255, 255, 255])
        self.assertEqual(len(template_dict["components"]), 2)
        self.assertFalse(template_dict["use_base_image"])

    @patch("PIL.Image.new")
    @patch("PIL.ImageDraw.Draw")
    def test_render(self, mock_draw, mock_new):
        "Test rendering a template."
        # Mock the image and draw objects
        mock_img = MagicMock()
        mock_new.return_value = mock_img
        mock_draw.return_value = MagicMock()

        # Mock component rendering
        for component in self.template.components:
            component.render = MagicMock()

        # Render the template
        result = self.template.render()

        # Check that the image was created with the correct size and color
        mock_new.assert_called_once_with("RGBA", (800, 600), (255, 255, 255, 255))

        # Check that each component was rendered
        for component in self.template.components:
            component.render.assert_called_once_with(mock_img, {})

        self.assertEqual(result, mock_img)


class TestTemplateEngine(unittest.TestCase):
    "Test case for the TemplateEngine class."

    def setUp(self):
        "Set up test fixtures."
        self.engine = TemplateEngine()
        self.template = Template("test_template", size=(800, 600))

    @patch("PIL.Image.new")
    @patch("PIL.ImageDraw.Draw")
    def test_render_template(self, mock_draw, mock_new):
        "Test rendering a template with the engine."
        # Mock the image and draw objects
        mock_img = MagicMock()
        mock_new.return_value = mock_img
        mock_draw.return_value = MagicMock()

        # Add a text component with a variable
        text_component = TextComponent("Hello, {{name}}!", position=(10, 10))
        self.template.add_component(text_component)

        # Render the template with variable substitution
        context = {"name": "World"}
        result = self.engine.render_template(self.template, context)

        # Check that the text was rendered with the variable substituted
        self.assertEqual(text_component.text, "Hello, World!")
        self.assertEqual(result, mock_img)

    def test_process_component_config(self):
        "Test processing a component configuration with variables."
        config = {
            "type": "text",
            "text": "Hello, {{name}}!",
            "position": {"x": 10, "y": 20},
            "font_size": 24,
            "color": [0, 0, 0, 255],
        }

        context = {"name": "World"}
        processed = self.engine._process_component_config(config, context)

        self.assertEqual(processed["text"], "Hello, World!")
        self.assertEqual(processed["position"]["x"], 10)
        self.assertEqual(processed["position"]["y"], 20)

    def test_process_component_config_nested(self):
        "Test processing a nested component configuration with variables."
        config = {
            "type": "container",
            "children": [
                {"type": "text", "text": "Hello, {{user.name}}!"},
                {"type": "image", "image_path": "{{user.avatar}}"},
            ],
        }

        context = {"user": {"name": "John", "avatar": "avatar.jpg"}}

        processed = self.engine._process_component_config(config, context)

        self.assertEqual(processed["children"][0]["text"], "Hello, John!")
        self.assertEqual(processed["children"][1]["image_path"], "avatar.jpg")


class TestTemplateRegistry(unittest.TestCase):
    "Test case for the TemplateRegistry class."

    def setUp(self):
        "Set up test fixtures."
        self.temp_dir = tempfile.mkdtemp()
        self.registry = TemplateRegistry(templates_dir=self.temp_dir)

        # Create a test template file
        self.template_data = {
            "name": "test_template",
            "size": {"width": 800, "height": 600},
            "background_color": [255, 255, 255, 255],
            "components": [
                {
                    "type": "text",
                    "text": "Hello, World!",
                    "position": {"x": 10, "y": 10},
                    "font_size": 24,
                    "color": [0, 0, 0, 255],
                }
            ],
        }

        self.template_path = os.path.join(self.temp_dir, "test_template.json")
        with open(self.template_path, "w") as f:
            json.dump(self.template_data, f)

    def tearDown(self):
        "Clean up test fixtures."
        import shutil

        shutil.rmtree(self.temp_dir)

    def test_register_template(self):
        "Test registering a template."
        template = Template.from_dict(self.template_data)
        self.registry.register_template("test_register", template)

        self.assertIn("test_register", self.registry.templates)
        self.assertEqual(self.registry.templates["test_register"], template)

    def test_get_template(self):
        "Test getting a template by name."
        template = Template.from_dict(self.template_data)
        self.registry.register_template("test_get", template)

        result = self.registry.get_template("test_get")
        self.assertEqual(result, template)

        # Test getting a non-existent template
        with self.assertRaises(ValueError):
            self.registry.get_template("non_existent")

    @patch("os.path.exists")
    @patch(
        "builtins.open",
        new_callable=mock_open,
        read_data=json.dumps(
            {
                "name": "test_load",
                "size": {"width": 800, "height": 600},
                "components": [],
            }
        ),
    )
    def test_load_template(self, mock_file, mock_exists):
        "Test loading a template from a file."
        mock_exists.return_value = True

        template = self.registry.load_template("test_load")

        self.assertEqual(template.name, "test_load")
        self.assertEqual(template.size, (800, 600))
        mock_file.assert_called_once()

    @patch("os.path.exists")
    def test_load_template_not_found(self, mock_exists):
        "Test loading a non-existent template file."
        mock_exists.return_value = False

        with self.assertRaises(FileNotFoundError):
            self.registry.load_template("non_existent")

    @patch("json.dump")
    @patch("builtins.open", new_callable=mock_open)
    def test_save_template(self, mock_file, mock_dump):
        "Test saving a template to a file."
        template = Template.from_dict(self.template_data)
        self.registry.save_template("test_save", template)

        mock_file.assert_called_once_with(
            os.path.join(self.temp_dir, "test_save.json"), "w", encoding="utf-8"
        )
        mock_dump.assert_called_once()

    def test_render_template(self):
        "Test rendering a template by name."
        template = Template.from_dict(self.template_data)
        self.registry.register_template("test_render", template)

        # Mock the template's render method
        template.render = MagicMock()

        # Render the template
        context = {"name": "World"}
        self.registry.render_template("test_render", context)

        # Check that the template's render method was called with the context
        template.render.assert_called_once_with(context)


class TestFontManager(unittest.TestCase):
    "Test case for the FontManager class."

    def setUp(self):
        "Set up test fixtures."
        self.temp_dir = tempfile.mkdtemp()
        self.font_manager = FontManager(fonts_dir=self.temp_dir)

        # Create a test font file
        self.font_path = os.path.join(self.temp_dir, "test_font.ttf")
        with open(self.font_path, "wb") as f:
            f.write(b"dummy font data")

    def tearDown(self):
        "Clean up test fixtures."
        import shutil

        shutil.rmtree(self.temp_dir)

    def test_add_font(self):
        "Test adding a font to the manager."
        self.font_manager.add_font("TestFont", self.font_path)

        self.assertIn("TestFont", self.font_manager.fonts)
        self.assertEqual(self.font_manager.fonts["TestFont"], self.font_path)

    @patch("PIL.ImageFont.truetype")
    def test_get_font(self, mock_truetype):
        "Test getting a font with a specific size."
        # Mock the ImageFont.truetype function
        mock_font = MagicMock()
        mock_truetype.return_value = mock_font

        # Add a test font
        self.font_manager.add_font("TestFont", self.font_path)

        # Get the font with a specific size
        font = self.font_manager.get_font("TestFont", 24)

        # Check that the font was loaded with the correct path and size
        mock_truetype.assert_called_once_with(self.font_path, 24)
        self.assertEqual(font, mock_font)

    @patch("PIL.ImageFont.truetype")
    def test_get_font_with_fallback(self, mock_truetype):
        "Test getting a font with fallback to default."
        # Mock the ImageFont.truetype function to raise an IOError for the first call
        mock_truetype.side_effect = [IOError("Font not found"), MagicMock()]

        # Add a test font as the default
        self.font_manager.default_font = "TestFont"
        self.font_manager.add_font("TestFont", self.font_path)

        # Try to get a non-existent font, should fall back to default
        font = self.font_manager.get_font("NonExistentFont", 24)

        # Check that the default font was loaded
        self.assertIsNotNone(font)
        mock_truetype.assert_called_with(self.font_path, 24)

    def test_load_fonts_from_directory(self):
        "Test loading fonts from a directory."
        # Create a subdirectory with font files
        os.makedirs(os.path.join(self.temp_dir, "subdir"))
        font1 = os.path.join(self.temp_dir, "font1.ttf")
        font2 = os.path.join(self.temp_dir, "subdir", "font2.ttf")

        with open(font1, "wb") as f:
            f.write(b"font1 data")
        with open(font2, "wb") as f:
            f.write(b"font2 data")

        # Load fonts from the directory
        self.font_manager.load_fonts_from_directory(self.temp_dir)

        # Check that both fonts were loaded with the correct names
        self.assertIn("font1", self.font_manager.fonts)
        self.assertIn("font2", self.font_manager.fonts)
        self.assertEqual(self.font_manager.fonts["font1"], font1)
        self.assertEqual(self.font_manager.fonts["font2"], font2)


if __name__ == "__main__":
    unittest.main()



"""
Font Manager - Manages and provides access to custom fonts
"""

import os
import logging
from typing import Dict, Optional, List
from PIL import ImageFont

# Set up logging
logging.basicConfig(
    level=logging.DEBUG, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)


class FontManager:
    """
    Manages font loading and provides access to custom fonts.
    """

    def __init__(self, font_dir: str = "fonts"):
        """
        Initialize the font manager.

        Args:
            font_dir: Directory containing font files
        """
        self.font_dir = font_dir
        self.fonts: Dict[str, str] = {}
        self._scan_fonts()

    def _scan_fonts(self) -> None:
        """
        Recursively scan the font directory and its subdirectories for available fonts.
        Fonts are stored with their base filename (without extension) as the key.
        """
        if not os.path.exists(self.font_dir):
            os.makedirs(self.font_dir, exist_ok=True)
            return

        for root, _, files in os.walk(self.font_dir):
            for filename in files:
                if filename.lower().endswith((".ttf", ".otf")):
                    font_name = os.path.splitext(filename)[0]
                    font_path = os.path.join(root, filename)
                    self.fonts[font_name] = font_path
                    logger.debug(f"Found font: {font_name} at {font_path}")

    def get_font_path(self, font_name: str) -> Optional[str]:
        """
        Get the path to a font file by name.

        Args:
            font_name: Name of the font (without extension)

        Returns:
            Path to the font file or None if not found
        """
        return self.fonts.get(font_name)

    def get_font(
        self, font_name: Optional[str] = None, size: int = 24
    ) -> ImageFont.FreeTypeFont:
        """
        Get a font by name and size.

        Args:
            font_name: Name of the font (without extension)
            size: Font size

        Returns:
            PIL ImageFont object
        """
        # If no font name is specified, use the first available font
        if font_name is None and self.fonts:
            font_path = next(iter(self.fonts.values()))
        elif font_name in self.fonts:
            font_path = self.fonts[font_name]
            logger.debug(f"Loading font: {font_name} from {font_path}")
        else:
            # If the requested font is not available, try to use a default system font
            try:
                # Common system fonts
                system_fonts = [
                    "/System/Library/Fonts/Helvetica.ttc",  # macOS
                    "/usr/share/fonts/truetype/dejavu/DejaVuSans.ttf",  # Linux
                    "C:\\Windows\\Fonts\\arial.ttf",  # Windows
                ]

                for font in system_fonts:
                    if os.path.exists(font):
                        return ImageFont.truetype(font, size)

                # If no system fonts are available, use the default font
                return ImageFont.load_default()
            except Exception as e:
                logger.warning(f"Failed to load system font: {e}")
                return ImageFont.load_default()

        try:
            return ImageFont.truetype(font_path, size)
        except Exception as e:
            logger.error(f"Failed to load font {font_path}: {e}")
            # Fallback to default font if loading fails
            return ImageFont.load_default()

    def list_fonts(self) -> List[str]:
        """
        Get a list of available font names.

        Returns:
            List of font names
        """
        return list(self.fonts.keys())


# Singleton instance for easy access
_instance = None


def get_font_manager(font_dir: str = "fonts") -> FontManager:
    """
    Get the singleton instance of the font manager.

    Args:
        font_dir: Directory containing font files

    Returns:
        FontManager instance
    """
    global _instance
    if _instance is None:
        _instance = FontManager(font_dir)
    return _instance



"""
Unit tests for the Dolze Templates components.
"""

import os
import unittest
import tempfile
from pathlib import Path
from unittest.mock import MagicMock, patch

# Add parent directory to path so we can import the package
import sys

sys.path.append(str(Path(__file__).parent.parent))

from dolze_templates.components import (
    Component,
    TextComponent,
    ImageComponent,
    CircleComponent,
    RectangleComponent,
    CTAButtonComponent,
    FooterComponent,
    create_component_from_config,
)
from dolze_templates.utils.validation import (
    validate_color,
    validate_position,
    validate_size,
)


class TestBaseComponent(unittest.TestCase):
    "Test case for the base Component class."

    def test_component_initialization(self):
        "Test that a component can be initialized with default values."
        component = Component("test")
        self.assertEqual(component.name, "test")
        self.assertEqual(component.position, (0, 0))
        self.assertIsNone(component.size)
        self.assertTrue(component.visible)

    def test_component_to_dict(self):
        "Test converting a component to a dictionary."
        component = Component("test", position=(10, 20), size=(100, 200), visible=False)
        component_dict = component.to_dict()

        self.assertEqual(component_dict["type"], "component")
        self.assertEqual(component_dict["position"], {"x": 10, "y": 20})
        self.assertEqual(component_dict["size"], {"width": 100, "height": 200})
        self.assertFalse(component_dict["visible"])


class TestTextComponent(unittest.TestCase):
    "Test case for the TextComponent class."

    def setUp(self):
        "Set up test fixtures."
        self.text = "Test Text"
        self.position = (50, 50)
        self.font_size = 24
        self.color = (0, 0, 0, 255)
        self.component = TextComponent(
            text=self.text,
            position=self.position,
            font_size=self.font_size,
            color=self.color,
        )

    def test_text_component_initialization(self):
        "Test that a text component can be initialized."
        self.assertEqual(self.component.text, self.text)
        self.assertEqual(self.component.position, self.position)
        self.assertEqual(self.component.font_size, self.font_size)
        self.assertEqual(self.component.color, self.color)
        self.assertEqual(self.component.alignment, "left")

    def test_text_component_to_dict(self):
        "Test converting a text component to a dictionary."
        component_dict = self.component.to_dict()

        self.assertEqual(component_dict["type"], "text")
        self.assertEqual(component_dict["text"], self.text)
        self.assertEqual(
            component_dict["position"], {"x": self.position[0], "y": self.position[1]}
        )
        self.assertEqual(component_dict["font_size"], self.font_size)
        self.assertEqual(component_dict["color"], list(self.color))

    @patch("PIL.ImageDraw.Draw.text")
    @patch("PIL.ImageDraw.Draw.multiline_textbbox")
    def test_render(self, mock_textbbox, mock_draw):
        "Test rendering a text component."
        # Mock the text bbox to return a fixed size
        mock_textbbox.return_value = (0, 0, 100, 50)

        # Create a mock image
        mock_image = MagicMock()
        mock_draw.return_value = MagicMock()

        # Render the component
        self.component.render(mock_image, {})

        # Check that the text was drawn
        mock_draw.return_value.text.assert_called_once()


class TestImageComponent(unittest.TestCase):
    "Test case for the ImageComponent class."

    def setUp(self):
        "Set up test fixtures."
        self.image_path = "test_image.jpg"
        self.position = (100, 100)
        self.size = (200, 200)
        self.component = ImageComponent(
            image_path=self.image_path, position=self.position, size=self.size
        )

    def test_image_component_initialization(self):
        "Test that an image component can be initialized."
        self.assertEqual(self.component.image_path, self.image_path)
        self.assertEqual(self.component.position, self.position)
        self.assertEqual(self.component.size, self.size)
        self.assertFalse(self.component.circle_crop)

    @patch("PIL.Image.open")
    def test_load_image_from_path(self, mock_open):
        "Test loading an image from a file path."
        # Mock the image open and size
        mock_img = MagicMock()
        mock_img.size = (400, 400)
        mock_open.return_value = mock_img

        # Test loading the image
        img = self.component._load_image()

        # Check that the image was opened and resized
        mock_open.assert_called_once_with(self.image_path)
        mock_img.resize.assert_called_once()
        self.assertIsNotNone(img)

    @patch("requests.get")
    def test_load_image_from_url(self, mock_get):
        "Test loading an image from a URL."
        # Create a component with a URL
        component = ImageComponent(
            image_url="http://example.com/image.jpg",
            position=self.position,
            size=self.size,
        )

        # Mock the response
        mock_response = MagicMock()
        mock_response.content = b"fake image data"
        mock_get.return_value = mock_response

        # Mock the Image.open to return a mock image
        with patch("PIL.Image.open") as mock_open:
            mock_img = MagicMock()
            mock_img.size = (400, 400)
            mock_open.return_value = mock_img

            # Test loading the image
            img = component._load_image()

            # Check that the image was loaded from the URL
            mock_get.assert_called_once_with(
                "http://example.com/image.jpg", stream=True
            )
            self.assertIsNotNone(img)


class TestComponentFactory(unittest.TestCase):
    "Test case for the component factory function."

    def test_create_text_component(self):
        "Test creating a text component from a config dictionary."
        config = {
            "type": "text",
            "text": "Hello, World!",
            "position": {"x": 10, "y": 20},
            "font_size": 24,
            "color": [0, 0, 0, 255],
        }

        component = create_component_from_config(config)

        self.assertIsInstance(component, TextComponent)
        self.assertEqual(component.text, "Hello, World!")
        self.assertEqual(component.position, (10, 20))
        self.assertEqual(component.font_size, 24)

    def test_create_image_component(self):
        "Test creating an image component from a config dictionary."
        config = {
            "type": "image",
            "image_path": "test.jpg",
            "position": {"x": 0, "y": 0},
            "size": {"width": 100, "height": 100},
        }

        component = create_component_from_config(config)

        self.assertIsInstance(component, ImageComponent)
        self.assertEqual(component.image_path, "test.jpg")
        self.assertEqual(component.size, (100, 100))

    def test_create_circle_component(self):
        "Test creating a circle component from a config dictionary."
        config = {
            "type": "circle",
            "position": {"x": 100, "y": 100},
            "radius": 50,
            "fill_color": [255, 0, 0, 255],
        }

        component = create_component_from_config(config)

        self.assertIsInstance(component, CircleComponent)
        self.assertEqual(component.position, (100, 100))
        self.assertEqual(component.radius, 50)
        self.assertEqual(component.fill_color, (255, 0, 0, 255))

    def test_create_rectangle_component(self):
        "Test creating a rectangle component from a config dictionary."
        config = {
            "type": "rectangle",
            "position": {"x": 10, "y": 10},
            "size": {"width": 100, "height": 50},
            "fill_color": [0, 0, 255, 255],
            "outline_color": [0, 0, 0, 255],
            "outline_width": 2,
        }

        component = create_component_from_config(config)

        self.assertIsInstance(component, RectangleComponent)
        self.assertEqual(component.position, (10, 10))
        self.assertEqual(component.size, (100, 50))
        self.assertEqual(component.fill_color, (0, 0, 255, 255))
        self.assertEqual(component.outline_color, (0, 0, 0, 255))
        self.assertEqual(component.outline_width, 2)

    def test_create_cta_button_component(self):
        "Test creating a CTA button component from a config dictionary."
        config = {
            "type": "cta_button",
            "text": "Click Me",
            "position": {"x": 10, "y": 10},
            "size": {"width": 200, "height": 50},
            "bg_color": [0, 123, 255, 255],
            "text_color": [255, 255, 255, 255],
            "corner_radius": 5,
        }

        component = create_component_from_config(config)

        self.assertIsInstance(component, CTAButtonComponent)
        self.assertEqual(component.text, "Click Me")
        self.assertEqual(component.position, (10, 10))
        self.assertEqual(component.size, (200, 50))
        self.assertEqual(component.bg_color, (0, 123, 255, 255))
        self.assertEqual(component.text_color, (255, 255, 255, 255))
        self.assertEqual(component.corner_radius, 5)

    def test_create_footer_component(self):
        "Test creating a footer component from a config dictionary."
        config = {
            "type": "footer",
            "text": "© 2023 My Company",
            "position": {"x": 0, "y": 580},
            "font_size": 14,
            "color": [255, 255, 255, 200],
            "bg_color": [0, 0, 0, 100],
            "padding": 10,
        }

        component = create_component_from_config(config)

        self.assertIsInstance(component, FooterComponent)
        self.assertEqual(component.text, "© 2023 My Company")
        self.assertEqual(component.position, (0, 580))
        self.assertEqual(component.font_size, 14)
        self.assertEqual(component.color, (255, 255, 255, 200))
        self.assertEqual(component.bg_color, (0, 0, 0, 100))
        self.assertEqual(component.padding, 10)


class TestValidationUtils(unittest.TestCase):
    "Test case for validation utility functions."

    def test_validate_color(self):
        "Test color validation."
        # Test hex color
        self.assertEqual(validate_color("#FF0000"), (255, 0, 0, 255))
        self.assertEqual(validate_color("#00FF00FF"), (0, 255, 0, 255))

        # Test RGB tuple
        self.assertEqual(validate_color((255, 0, 0)), (255, 0, 0, 255))

        # Test RGBA tuple
        self.assertEqual(validate_color((0, 255, 0, 128)), (0, 255, 0, 128))

        # Test invalid colors
        with self.assertRaises(ValueError):
            validate_color("not a color")

        with self.assertRaises(ValueError):
            validate_color((300, 0, 0))  # Invalid RGB value

    def test_validate_position(self):
        "Test position validation."
        # Test dict position
        self.assertEqual(validate_position({"x": 10, "y": 20}), (10, 20))

        # Test tuple position
        self.assertEqual(validate_position((30, 40)), (30, 40))

        # Test invalid positions
        with self.assertRaises(ValueError):
            validate_position("not a position")

        with self.assertRaises(ValueError):
            validate_position((1, 2, 3))  # Too many values

    def test_validate_size(self):
        "Test size validation."
        # Test dict size
        self.assertEqual(validate_size({"width": 100, "height": 200}), (100, 200))

        # Test tuple size
        self.assertEqual(validate_size((300, 400)), (300, 400))

        # Test invalid sizes
        with self.assertRaises(ValueError):
            validate_size({"width": -10, "height": 100})  # Negative width

        with self.assertRaises(ValueError):
            validate_size((1, 2, 3))  # Too many values


if __name__ == "__main__":
    unittest.main()
